"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var gesturehandler_common_1 = require("./gesturehandler.common");
exports.GestureHandlerStateEvent = gesturehandler_common_1.GestureHandlerStateEvent;
exports.GestureHandlerTouchEvent = gesturehandler_common_1.GestureHandlerTouchEvent;
exports.GestureState = gesturehandler_common_1.GestureState;
exports.HandlerType = gesturehandler_common_1.HandlerType;
exports.ViewDisposeEvent = gesturehandler_common_1.ViewDisposeEvent;
exports.ViewInitEvent = gesturehandler_common_1.ViewInitEvent;
var gestures_override_1 = require("./gestures_override");
var utils_1 = require("@nativescript/core/utils/utils");
var application_1 = require("@nativescript/core/application");
var page_1 = require("@nativescript/core/ui/page");
var gestures_1 = require("@nativescript/core/ui/gestures");
var PageLayout;
var PageGestureExtended = /** @class */ (function (_super) {
    __extends(PageGestureExtended, _super);
    function PageGestureExtended() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PageGestureExtended.prototype.initNativeView = function () {
        this.nativeView.initialize();
    };
    PageGestureExtended.prototype.disposeNativeView = function () {
        this.nativeView.tearDown();
    };
    Object.defineProperty(PageGestureExtended.prototype, "registry", {
        get: function () {
            return this.nativeView && this.nativeView.registry();
        },
        enumerable: true,
        configurable: true
    });
    return PageGestureExtended;
}(page_1.Page));
var installed = false;
function install(overrideNGestures) {
    if (overrideNGestures === void 0) { overrideNGestures = false; }
    if (installed) {
        return;
    }
    installed = true;
    gesturehandler_common_1.install();
    var NSPage = require('@nativescript/core/ui/page').Page;
    NSPage.prototype.createNativeView = function () {
        if (!PageLayout) {
            PageLayout = com.nativescript.gesturehandler.PageLayout;
        }
        var layout = new PageLayout(this._context);
        this.gestureRegistry = layout.registry;
        return layout;
    };
    gesturehandler_common_1.applyMixins(NSPage, [PageGestureExtended]);
    if (overrideNGestures === true) {
        var NSView = require('@nativescript/core/ui/core/view').View;
        NSView.prototype._observe = function (type, callback, thisArg) {
            if (!this._gestureObservers[type]) {
                this._gestureObservers[type] = [];
            }
            this._gestureObservers[type].push(gestures_override_1.observe(this, type, callback, thisArg));
            if (type & gestures_1.GestureTypes.touch && this.isLoaded && !this.touchListenerIsSet) {
                this.setOnTouchListener();
            }
        };
        NSView.prototype.setOnTouchListener = function () {
            var _this = this;
            if (!this.nativeViewProtected || !this.getGestureObservers(gestures_1.GestureTypes.touch)) {
                return;
            }
            // do not set noop listener that handles the event (disabled listener) if IsUserInteractionEnabled is
            // false as we might need the ability for the event to pass through to a parent view
            this.touchListener =
                this.touchListener ||
                    new android.view.View.OnTouchListener({
                        onTouch: function (view, event) {
                            _this.handleGestureTouch(event);
                            var nativeView = _this.nativeViewProtected;
                            if (!nativeView || !nativeView.onTouchEvent) {
                                return false;
                            }
                            return nativeView.onTouchEvent(event);
                        },
                    });
            this.nativeViewProtected.setOnTouchListener(this.touchListener);
            this.touchListenerIsSet = true;
            if (this.nativeViewProtected.setClickable) {
                this.nativeViewProtected.setClickable(this.isUserInteractionEnabled);
            }
        };
    }
}
exports.install = install;
var KEY_HIT_SLOP_LEFT = 'left';
var KEY_HIT_SLOP_TOP = 'top';
var KEY_HIT_SLOP_RIGHT = 'right';
var KEY_HIT_SLOP_BOTTOM = 'bottom';
var KEY_HIT_SLOP_VERTICAL = 'vertical';
var KEY_HIT_SLOP_HORIZONTAL = 'horizontal';
var KEY_HIT_SLOP_WIDTH = 'width';
var KEY_HIT_SLOP_HEIGHT = 'height';
var Handler = /** @class */ (function (_super) {
    __extends(Handler, _super);
    function Handler() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.tag = 0;
        return _this;
    }
    Handler.prototype.getExtraData = function (handler) {
        var numberOfPointers = handler.getNumberOfPointers();
        var positions = [];
        for (var index_1 = 0; index_1 < numberOfPointers; index_1++) {
            positions.push(handler.getXAtIndex(index_1));
            positions.push(handler.getYAtIndex(index_1));
        }
        return {
            // x: layout.toDeviceIndependentPixels(handler.getX()),
            // y: layout.toDeviceIndependentPixels(handler.getY()),
            positions: positions,
            numberOfPointers: numberOfPointers,
        };
    };
    Handler.prototype.initNativeView = function (native, options) {
        _super.prototype.initNativeView.call(this, native, options);
        this.native.setTag(this.tag);
        this.touchListener = new com.swmansion.gesturehandler.OnTouchEventListener({
            onTouchEvent: this.onTouchEvent.bind(this),
            onStateChange: this.onStateChange.bind(this),
        });
        native.setOnTouchEventListener(this.touchListener);
        this.manager.get().configureInteractions(this, options);
    };
    Handler.prototype.disposeNativeView = function () {
        this.native.setInteractionController(null);
        this.native.setOnTouchEventListener(null);
        this.touchListener = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Handler.prototype.onTouchEvent = function (handler, event) {
        var view = handler.getView();
        this.notify({
            eventName: gesturehandler_common_1.GestureHandlerTouchEvent,
            object: this,
            data: {
                state: handler.getState(),
                android: view,
                extraData: this.getExtraData(handler),
                view: view.nsView ? view.nsView.get() : null,
            },
        });
    };
    Handler.prototype.onStateChange = function (handler, state, prevState) {
        var view = handler.getView();
        this.notify({
            eventName: gesturehandler_common_1.GestureHandlerStateEvent,
            object: this,
            data: {
                state: state,
                prevState: prevState,
                android: view,
                extraData: this.getExtraData(handler),
                view: view.nsView ? view.nsView.get() : null,
            },
        });
    };
    Handler.prototype.setTag = function (tag) {
        this.tag = tag;
        if (this.native) {
            this.native.setTag(tag);
        }
    };
    Handler.prototype.getTag = function () {
        return this.tag;
        // return this.getNative().getTag();
    };
    Handler.prototype.getView = function () {
        return this.getNative().getView();
    };
    Handler.prototype.cancel = function () {
        return this.getNative().cancel();
    };
    Handler.prototype.attachToView = function (view) {
        this.manager.get().attachGestureHandler(this, view);
    };
    Handler.prototype.detachFromView = function (view) {
        this.manager.get().detachGestureHandler(this, view);
    };
    __decorate([
        gesturehandler_common_1.nativeProperty({
            nativeSetterName: 'setHitSlop',
            nativegetterName: 'getHitSlop',
            converter: {
                toNative: function (value) {
                    var HIT_SLOP_NONE = com.swmansion.gesturehandler.GestureHandler.HIT_SLOP_NONE;
                    if (typeof value === 'number') {
                        var hitSlop = utils_1.layout.toDevicePixels(value);
                        return [hitSlop, hitSlop, hitSlop, hitSlop, HIT_SLOP_NONE, HIT_SLOP_NONE];
                    }
                    else {
                        var left = HIT_SLOP_NONE, top_1 = HIT_SLOP_NONE, right = HIT_SLOP_NONE, bottom = HIT_SLOP_NONE;
                        var width = HIT_SLOP_NONE, height = HIT_SLOP_NONE;
                        if (value.hasOwnProperty(KEY_HIT_SLOP_HORIZONTAL)) {
                            var horizontalPad = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_HORIZONTAL]);
                            left = right = horizontalPad;
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_VERTICAL)) {
                            var verticalPad = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_VERTICAL]);
                            top_1 = bottom = verticalPad;
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_LEFT)) {
                            left = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_LEFT]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_TOP)) {
                            top_1 = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_TOP]);
                        }
                        if (value.hasKey(KEY_HIT_SLOP_RIGHT)) {
                            right = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_RIGHT]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_BOTTOM)) {
                            bottom = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_BOTTOM]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_WIDTH)) {
                            width = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_WIDTH]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_HEIGHT)) {
                            height = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_HEIGHT]);
                        }
                        return [left, top_1, right, bottom, width, height];
                    }
                },
            },
        }),
        __metadata("design:type", Object)
    ], Handler.prototype, "hitSlop", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], Handler.prototype, "enabled", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], Handler.prototype, "shouldCancelWhenOutside", void 0);
    return Handler;
}(gesturehandler_common_1.BaseNative));
exports.Handler = Handler;
var TapGestureHandler = /** @class */ (function (_super) {
    __extends(TapGestureHandler, _super);
    function TapGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TapGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.TapGestureHandler();
    };
    TapGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            x: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionX()),
            y: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionY()),
            absoluteX: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionX()),
            absoluteY: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionY()),
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "numberOfTaps", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDurationMs", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDelayMs", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setMaxDx' }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDeltaX", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setMaxDy' }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDeltaY", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDist", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setMinNumberOfPointers' }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "minPointers", void 0);
    return TapGestureHandler;
}(Handler));
exports.TapGestureHandler = TapGestureHandler;
var PanGestureHandler = /** @class */ (function (_super) {
    __extends(PanGestureHandler, _super);
    function PanGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PanGestureHandler.prototype.createNative = function (options) {
        var context = application_1.android.context;
        return new com.swmansion.gesturehandler.PanGestureHandler(context);
    };
    PanGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            x: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionX()),
            y: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionY()),
            absoluteX: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionX()),
            absoluteY: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionY()),
            translationX: utils_1.layout.toDeviceIndependentPixels(handler.getTranslationX()),
            translationY: utils_1.layout.toDeviceIndependentPixels(handler.getTranslationY()),
            velocityX: utils_1.layout.toDeviceIndependentPixels(handler.getVelocityX()),
            velocityY: utils_1.layout.toDeviceIndependentPixels(handler.getVelocityY()),
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minDist", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetXStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetXEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetXStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetXEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetYStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetYEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetYStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetYEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minVelocity", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minVelocityX", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minVelocityY", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minPointers", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "maxPointers", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setAverageTouches' }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "avgTouches", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "numberOfPointers", void 0);
    return PanGestureHandler;
}(Handler));
exports.PanGestureHandler = PanGestureHandler;
var PinchGestureHandler = /** @class */ (function (_super) {
    __extends(PinchGestureHandler, _super);
    function PinchGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PinchGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.PinchGestureHandler();
    };
    PinchGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            scale: handler.getScale(),
            focalX: utils_1.layout.toDeviceIndependentPixels(handler.getFocalPointX()),
            focalY: utils_1.layout.toDeviceIndependentPixels(handler.getFocalPointY()),
            velocity: handler.getVelocity(),
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PinchGestureHandler.prototype, "minSpan", void 0);
    return PinchGestureHandler;
}(Handler));
exports.PinchGestureHandler = PinchGestureHandler;
function directionToString(direction) {
    switch (direction) {
        case com.swmansion.gesturehandler.GestureHandler.DIRECTION_RIGHT:
            return 'right';
        case com.swmansion.gesturehandler.GestureHandler.DIRECTION_UP:
            return 'up';
        case com.swmansion.gesturehandler.GestureHandler.DIRECTION_DOWN:
            return 'down';
        default:
            return 'left';
    }
}
function directionFromString(direction) {
    switch (direction) {
        case 'right':
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_RIGHT;
        case 'up':
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_UP;
        case 'down':
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_DOWN;
        default:
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_LEFT;
    }
}
var FlingGestureHandler = /** @class */ (function (_super) {
    __extends(FlingGestureHandler, _super);
    function FlingGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FlingGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.FlingGestureHandler();
    };
    FlingGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            direction: directionToString(handler.getRecognizedDirection()),
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], FlingGestureHandler.prototype, "numberOfPointers", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: directionToString, toNative: directionFromString } }),
        __metadata("design:type", String)
    ], FlingGestureHandler.prototype, "direction", void 0);
    return FlingGestureHandler;
}(Handler));
exports.FlingGestureHandler = FlingGestureHandler;
var LongPressGestureHandler = /** @class */ (function (_super) {
    __extends(LongPressGestureHandler, _super);
    function LongPressGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LongPressGestureHandler.prototype.createNative = function (options) {
        var context = application_1.android.context;
        return new com.swmansion.gesturehandler.LongPressGestureHandler(context);
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], LongPressGestureHandler.prototype, "minDurationMs", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], LongPressGestureHandler.prototype, "maxDist", void 0);
    return LongPressGestureHandler;
}(Handler));
exports.LongPressGestureHandler = LongPressGestureHandler;
var RotationGestureHandler = /** @class */ (function (_super) {
    __extends(RotationGestureHandler, _super);
    function RotationGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RotationGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.RotationGestureHandler();
    };
    RotationGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            rotation: handler.getRotation(),
            anchorX: utils_1.layout.toDeviceIndependentPixels(handler.getAnchorX()),
            anchorY: utils_1.layout.toDeviceIndependentPixels(handler.getAnchorY()),
            velocity: handler.getVelocity(),
        });
    };
    return RotationGestureHandler;
}(Handler));
exports.RotationGestureHandler = RotationGestureHandler;
var NativeViewGestureHandler = /** @class */ (function (_super) {
    __extends(NativeViewGestureHandler, _super);
    function NativeViewGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NativeViewGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.NativeViewGestureHandler();
    };
    NativeViewGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            pointerInside: handler.isWithinBounds(),
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], NativeViewGestureHandler.prototype, "shouldActivateOnStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], NativeViewGestureHandler.prototype, "disallowInterruption", void 0);
    return NativeViewGestureHandler;
}(Handler));
exports.NativeViewGestureHandler = NativeViewGestureHandler;
var Manager = /** @class */ (function (_super) {
    __extends(Manager, _super);
    function Manager() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.viewListeners = new Map();
        return _this;
    }
    Manager.prototype.configureInteractions = function (handler, options) {
        if (options === void 0) { options = {}; }
        this.interactionManager.configureInteractions(handler.getNative(), options.waitFor, options.simultaneousHandlers);
    };
    Object.defineProperty(Manager.prototype, "interactionManager", {
        get: function () {
            if (!this._interactionManager) {
                // initGestureHandlerInteractionController();
                this._interactionManager = new com.nativescript.gesturehandler.GestureHandlerInteractionController();
            }
            return this._interactionManager;
        },
        enumerable: true,
        configurable: true
    });
    Manager.getInstance = function () {
        if (!Manager.sManager) {
            Manager.sManager = new Manager();
        }
        return Manager.sManager;
    };
    Manager.prototype.createGestureHandler = function (handlerName, handlerTag, config) {
        var handler = null;
        switch (handlerName) {
            case 'tap':
                handler = new TapGestureHandler(config);
                break;
            case 'pan':
                handler = new PanGestureHandler(config);
                break;
            case 'nativeView':
                handler = new NativeViewGestureHandler(config);
                break;
            case 'pinch':
                handler = new PinchGestureHandler(config);
                break;
            case 'fling':
                handler = new FlingGestureHandler(config);
                break;
            case 'rotation':
                handler = new RotationGestureHandler(config);
                break;
            case 'longPress':
                handler = new LongPressGestureHandler(config);
                break;
        }
        if (handler) {
            handler.manager = new WeakRef(this);
            handler.setTag(handlerTag);
        }
        return handler;
    };
    Manager.prototype.attachGestureHandlerToView = function (handler, view) {
        var nHandler = handler.getNative();
        if (nHandler) {
            var page = view.page;
            if (page) {
                var registry = page.registry;
                if (registry) {
                    registry.registerHandler(nHandler);
                    registry.attachHandlerToView(nHandler.getTag(), view.nativeView);
                }
            }
            else {
                throw new Error('a page is needed to attach a gesture');
            }
        }
    };
    Manager.prototype.detachGestureHandlerFromView = function (handler, view) {
        var nHandler = handler.getNative();
        if (nHandler) {
            var registry = view.page && view.page.registry;
            if (registry) {
                registry.dropHandler(handler.getNative());
            }
        }
    };
    Manager.prototype.attachGestureHandler = function (handler, view) {
        var _this = this;
        if (view.nativeView) {
            this.attachGestureHandlerToView(handler, view);
        }
        var onInit = function () { return _this.attachGestureHandlerToView(handler, view); };
        var onDispose = function () { return _this.detachGestureHandlerFromView(handler, view); };
        view.on(gesturehandler_common_1.ViewInitEvent, onInit, this);
        view.on(gesturehandler_common_1.ViewDisposeEvent, onDispose, this);
        var viewListeners = this.viewListeners.get(view);
        if (!viewListeners) {
            viewListeners = new Map();
            this.viewListeners.set(view, viewListeners);
        }
        viewListeners.set(handler.getTag(), {
            init: onInit,
            dispose: onDispose,
        });
    };
    Manager.prototype.detachGestureHandler = function (handler, view) {
        if (view) {
            var viewListeners = this.viewListeners.get(view);
            if (viewListeners) {
                var listeners_1 = viewListeners.get(handler.getTag());
                if (listeners_1) {
                    view.off(gesturehandler_common_1.ViewInitEvent, listeners_1.init, this);
                    view.off(gesturehandler_common_1.ViewDisposeEvent, function () { return listeners_1.dispose; }, this);
                    viewListeners.delete(handler.getTag());
                    if (viewListeners.size === 0) {
                        this.viewListeners.delete(view);
                    }
                }
            }
        }
        this.detachGestureHandlerFromView(handler, view);
    };
    return Manager;
}(gesturehandler_common_1.Manager));
exports.Manager = Manager;
//# sourceMappingURL=gesturehandler.android.js.map